#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Example dbus server for gportal.

Example dbus server class for the gportal/gterminal frontend.
"""
# Copyright (C) 2008-2010  James Shubin, McGill University
# Written for McGill University by James Shubin <purpleidea@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import _fixpath				# fix working directory in this example
import dbus
import dbus.mainloop.glib
import gobject
# try the source directory first, and if this fails then see if it's installed.
try:
	try:
		# XXX: this is supposed to work, but it never does! WHY ?
		from src import gportal
	except ImportError, e:
		# XXX: hack the import...
		import sys
		sys.path.append('src/')
		import gportal
		sys.path.pop()
		#raise ImportError
except ImportError, e:
	from aquiline import gportal

_ = lambda x: x			# add fake gettext function until i fix up i18n
__all__ = ('_service', '_interface', '_path', 'example_gserver')

# these three values can be anything that works for your dbus server
_service = 'ca.mcgill.cs.dazzle.Example_Service.server'
_path = '/Some_Path'
_interface = 'ca.mcgill.cs.dazzle.Some_Service.Some_Interface'

# magic interface gportal will look for
GINTERFACE = 'ca.mcgill.cs.dazzle.gportal.ginterface'
# you can get this magic value from gportal global constants
GINTERFACE = gportal.DBUS.interface


# XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX
# hack to be able to use this file to run two different servers...
# run this file argumentless and with arguments to trigger different servers.
GINTERFACE1 = 'ca.mcgill.cs.dazzle.gportal.ginterface'
GINTERFACE2 = 'ca.mcgill.cs.dazzle.gportal.ginterfaceX'
import sys
if len(sys.argv) > 1:
	_service = 'ca.mcgill.cs.dazzle.Example_ServiceTWO.server'
	_path = '/Some_Other_Path'
	GINTERFACE1 = 'ca.mcgill.cs.dazzle.gportal.ginterfaceX'	# not magic
	GINTERFACE2 = 'ca.mcgill.cs.dazzle.gportal.ginterface'	# magic
else:
	print "run: %s TWO to run a second g_dbus_example_server" % sys.argv[0]
# XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX


class example_gserver(gportal.dbus_service):
	# the initialization values can be specified here, to avoid using init 
	bus=dbus.SessionBus()
	#busName='this.is.an.example.bus.name'
	busname=_service
	#path='/this/is/some/path'
	path=_path

	"""example dbus server class for gportal."""
	# METHODS #############################################################
	@dbus.service.method(GINTERFACE1, in_signature='as', out_signature='s')
	def do_listargs(self, argv=()):
		"""Calls a hello method, then a signal, and prints/returns all
		these results including the incoming arguments in reverse."""
		msg = self.SomeHelloMethod('World')	# call any method

		result = ''
		result += 'Received: %s\n' % argv
		result += 'Msg: %s\n' % msg
		print result
		return result


	@dbus.service.method(GINTERFACE1, in_signature='s', out_signature='s')
	def Hello(self, name):
		print 'Hello, %s! (printed)' % name
		return 'Hello, %s! (returned)' % name


	@dbus.service.method(GINTERFACE2, in_signature='as', out_signature='s')
	def do_listargsrev(self, argv=()):
		"""Returns args in reverse."""
		print ('Received: %s' % list(reversed(argv)))
		return ('Received: %s' % list(reversed(argv)))


	@dbus.service.method(GINTERFACE, in_signature='as', out_signature='s')
	def do_message_test(self, argv=()):
		"""Causes a signal to be emitted, to display a message."""
		self._message(message='i am a SPECIAL message!', lifetime=9, name='something')
		print _('Sending message signal.')
		return ''


	@dbus.service.method(_interface, in_signature='s', out_signature='s')
	def SomeHelloMethod(self, name):
		"""This is some random method with the general server interface
		and doesn't show up or get exported to the gportal. It can be
		called by any method in this class, or through a regular dbus
		call."""
		print 'Hello, %s! (printed)' % name
		return 'Hello, %s! (returned)' % name


def main():
	# make dbus happy
	dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
	try:
		# create the dbus object
		obj = example_gserver()
	except dbus.exceptions.DBusException, e:
		raise e
		return

	mainloop = gobject.MainLoop()
	mainloop.run()

if __name__ == '__main__':
	main()

